/* the sidebar */
.tweak-categories {
    background-color: #ebedeb;
}
.tweak-category {
    padding: 10px;
}

/* the container and tweaks in a group */
.tweak-group {
    background-color: @theme_bg_color;
}

.tweak {
    padding-top: 3px;
    background-color: @theme_bg_color;
}
.tweak:hover {
    background-color: @theme_bg_color;
}
.tweak.title {
    padding-top: 10px;
}

/* individual tweak theme changes */
.tweak.title#title-theme {
    padding-top: 3px;
}
.tweak#hinting,
.tweak#text-scaling-factor {
    padding-top: 20px;
}

.tweak-group-white,
.tweak-white,
.tweak-white:hover {
    background-color: white;
}

/* NOT WORKING 
.main-container {
    padding: 20px;
}
*/

.search-button {
    padding-top: 5px;
    padding-bottom: 5px;
}