DATA_DIR = "/usr/local/share"
PKG_DATA_DIR = "/usr/local/share/gnome-tweak-tool"
GSETTINGS_SCHEMA_DIR = "/usr/local/share/glib-2.0/schemas"
TWEAK_DIR = "/usr/local/lib/python2.7/dist-packages/gtweak/tweaks"
VERSION = "3.9.90.1"
LOCALE_DIR = "/usr/local/share/locale"
